@@ -8,7 +8,7 @@ __metaclass__ = type
 
 ANSIBLE_METADATA = {'metadata_version': '1.1',
                     'status': ['stableinterface'],
-                    'supported_by': 'certified'}
+                    'supported_by': 'community'}
 
 
 DOCUMENTATION = '''
@@ -22,8 +22,6 @@ options:
   iam_type:
     description:
       - Type of IAM resource
-    required: true
-    default: null
     choices: ["user", "group", "role"]
   name:
     description:
@@ -32,47 +30,34 @@ options:
   new_name:
     description:
       - When state is update, will replace name with new_name on IAM resource
-    required: false
-    default: null
   new_path:
     description:
       - When state is update, will replace the path with new_path on the IAM resource
-    required: false
-    default: null
   state:
     description:
       - Whether to create, delete or update the IAM resource. Note, roles cannot be updated.
     required: true
-    default: null
     choices: [ "present", "absent", "update" ]
   path:
     description:
       - When creating or updating, specify the desired path of the resource. If state is present,
         it will replace the current path to match what is passed in when they do not match.
-    required: false
     default: "/"
   trust_policy:
     description:
       - The inline (JSON or YAML) trust policy document that grants an entity permission to assume the role. Mutually exclusive with C(trust_policy_filepath).
-    required: false
-    default: null
     version_added: "2.2"
   trust_policy_filepath:
     description:
       - The path to the trust policy document that grants an entity permission to assume the role. Mutually exclusive with C(trust_policy).
-    required: false
-    default: null
     version_added: "2.2"
   access_key_state:
     description:
       - When type is user, it creates, removes, deactivates or activates a user's access key(s). Note that actions apply only to keys specified.
-    required: false
-    default: null
     choices: [ "create", "remove", "active", "inactive"]
   key_count:
     description:
       - When access_key_state is create it will ensure this quantity of keys are present. Defaults to 1.
-    required: false
     default: '1'
   access_key_ids:
     description:
@@ -80,15 +65,10 @@ options:
   groups:
     description:
       - A list of groups the user should belong to. When update, will gracefully remove groups not listed.
-    required: false
-    default: null
   password:
     description:
       - When type is user and state is present, define the users login password. Also works with update. Note that always returns changed.
-    required: false
-    default: null
   update_password:
-    required: false
     default: always
     choices: ['always', 'on_create']
     description:
@@ -114,7 +94,7 @@ tasks:
     state: present
     password: "{{ temp_pass }}"
     access_key_state: create
-  with_items:
+  loop:
     - jcleese
     - mpython
 
@@ -126,7 +106,7 @@ task:
     iam_type: group
     name: "{{ item }}"
     state: present
-  with_items:
+  loop:
      - Mario
      - Luigi
   register: new_groups
@@ -137,7 +117,7 @@ task:
     name: jdavila
     state: update
     groups: "{{ item.created_group.group_name }}"
-  with_items: "{{ new_groups.results }}"
+  loop: "{{ new_groups.results }}"
 
 # Example of role with custom trust policy for Lambda service
 - name: Create IAM role with custom trust relationship
@@ -157,7 +137,7 @@ task:
 RETURN = '''
 role_result:
     description: the IAM.role dict returned by Boto
-    type: string
+    type: str
     returned: if iam_type=role and state=present
     sample: {
                 "arn": "arn:aws:iam::A1B2C3D4E5F6:role/my-new-role",
@@ -274,7 +254,7 @@ def delete_dependencies_first(module, iam, name):
         changed = True
     except boto.exception.BotoServerError as err:
         error_msg = boto_exception(err)
-        if 'Cannot find Login Profile' not in error_msg:
+        if 'Login Profile for User ' + name + ' cannot be found.' not in error_msg:
             module.fail_json(changed=changed, msg="Failed to delete login profile: %s" % err, exception=traceback.format_exc())
 
     # try to detach policies
@@ -285,9 +265,9 @@ def delete_dependencies_first(module, iam, name):
     except boto.exception.BotoServerError as err:
         error_msg = boto_exception(err)
         if 'must detach all policies first' in error_msg:
-            module.fail_json(changed=changed, msg="All inline polices have been removed. Though it appears"
+            module.fail_json(changed=changed, msg="All inline policies have been removed. Though it appears"
                                                   "that %s has Managed Polices. This is not "
-                                                  "currently supported by boto. Please detach the polices "
+                                                  "currently supported by boto. Please detach the policies "
                                                   "through the console and try again." % name)
         module.fail_json(changed=changed, msg="Failed to delete policies: %s" % err, exception=traceback.format_exc())
 
@@ -514,9 +494,9 @@ def delete_group(module=None, iam=None, name=None):
             except boto.exception.BotoServerError as err:
                 error_msg = boto_exception(err)
                 if ('must delete policies first') in error_msg:
-                    module.fail_json(changed=changed, msg="All inline polices have been removed. Though it appears"
+                    module.fail_json(changed=changed, msg="All inline policies have been removed. Though it appears"
                                                           "that %s has Managed Polices. This is not "
-                                                          "currently supported by boto. Please detach the polices "
+                                                          "currently supported by boto. Please detach the policies "
                                                           "through the console and try again." % name)
                 else:
                     module.fail_json(changed=changed, msg=str(error_msg))
@@ -598,9 +578,9 @@ def delete_role(module, iam, name, role_list, prof_list):
                 except boto.exception.BotoServerError as err:
                     error_msg = boto_exception(err)
                     if ('must detach all policies first') in error_msg:
-                        module.fail_json(changed=changed, msg="All inline polices have been removed. Though it appears"
+                        module.fail_json(changed=changed, msg="All inline policies have been removed. Though it appears"
                                                               "that %s has Managed Polices. This is not "
-                                                              "currently supported by boto. Please detach the polices "
+                                                              "currently supported by boto. Please detach the policies "
                                                               "through the console and try again." % name)
                     else:
                         module.fail_json(changed=changed, msg=str(err))
